/*
   Rotating 3D Cube Demo.
   Created by the Programming Ace.
   www.UltimateGameProgramming.com

   The purpose of this demo is to create a 3D cube in OpenGL.  We will use
   the OpenGL function glRotatef() to rotate our scene.  glRotatef() has
   the prototype:

   void glRotatef(GLfloat angle, GLfloat x, GLfloat y, GLfloat z);

   Calling this function before you draw an object will allow the object
   to be rotated along any axis by the amount you specify in angle.

*/


// Demo: Rotating 3D Cube.


#define WIN32_LEAN_AND_MEAN      // This trims down the libraries.
#define VC_LEANMEAN

#include<windows.h>              // The windows header file.
#include<gl/gl.h>                // Standard opengl include.
#include<gl/glu.h>               // Opengl utilities.
#include<gl/glaux.h>             // auxiliary functions.


// Function Prototypes
void RenderScene();
bool InitializeGL();
void SetupPixelFormat(HDC hDC);


float angle = 0.0f;              // This will hold the cube's angle.
HDC g_HDC;                       // Global device context.


// Light properties for the diffuse light, specular light, and light position.
float diffuseLight[] = {0.8f, 0.8f, 0.8f, 1.0f};
float specularLight[] = {1.0f, 1.0f, 1.0f, 1.0f};
float LightPosition[] = {1.0f, 0.0f, 10.0f, 1.0f};


void SetupPixelFormat(HDC hDC)
{
   int nPixelFormat;

   static PIXELFORMATDESCRIPTOR pfd = {
         sizeof(PIXELFORMATDESCRIPTOR),   // size of structure.
         1,                               // always 1.
         PFD_DRAW_TO_WINDOW |             // support window
         PFD_SUPPORT_OPENGL |             // support OpenGl
         PFD_DOUBLEBUFFER,                // support double buffering
         PFD_TYPE_RGBA,                   // support RGBA
         16,                              // bit color mode
         0, 0, 0, 0, 0, 0,                // ignore color bits
         0,                               // no alpha buffer
         0,                               // ignore shift bit
         0,                               // no accumulation buffer
         0, 0, 0, 0,                      // ignore accumulation bits.
         16,                              // number of depth buffer bits.
         0,                               // number of stencil buffer bits.
         0,                               // 0 means no auxiliary buffer
         PFD_MAIN_PLANE,                  // The main drawing plane
         0,                               // this is reserved
         0, 0, 0 };                       // layer masks ignored.

   // this chooses the best pixel format and returns index.
   nPixelFormat = ChoosePixelFormat(hDC, &pfd);

   // This set pixel format to device context.
   SetPixelFormat(hDC, nPixelFormat, &pfd);
}


LRESULT CALLBACK WndProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
{
   static HGLRC hRC;                      // Rendering context.
   static HDC hDC;                        // Device context.
   int width, height;                     // The window width and height.

   switch(message)
      {
         case WM_CREATE:                  // Windows creation.
            hDC = GetDC(hwnd);            // This gets the device context for our window.
            g_HDC = hDC;                  // Assigns the global device context to this one.
            SetupPixelFormat(hDC);        // Call the pixel format function.

            hRC = wglCreateContext(hDC);  // Creates the rendering context.
            wglMakeCurrent(hDC, hRC);     // Makes the rendering context.
            return 0;
            break;

         case WM_CLOSE:                   // Close message.
         case WM_DESTROY:
            wglMakeCurrent(hDC, NULL);
            wglDeleteContext(hRC);        // Deletes the rendering context.

            PostQuitMessage(0);           // Says close the program.
            return 0;
            break;

         case WM_SIZE:                    // re-size message.
            height = HIWORD(lParam);      // This gets the height of the window.
            width = LOWORD(lParam);       // This gets the width of the window.

            if(height==0)                 // we don't want it to be possible for a
               {                          // height of 0.  If it is 0 me make it 1.
                  height = 1;
               }

            glViewport(0, 0, width, height);// resets the viewport to new dimensions.
            glMatrixMode(GL_PROJECTION);    // Sets the projection matrix.
            glLoadIdentity();               // Reset the modelview matrix.

            // calculate the aspect ratio of the window.
            gluPerspective(45.0f, (GLfloat)width/(GLfloat)height, 0.1f, 1000.0f);

            glMatrixMode(GL_MODELVIEW);     // Sets the projection matrix.
            glLoadIdentity();               // Reset the modelview matrix.

            return 0;
            break;

         case WM_KEYDOWN:
		      switch(wParam) 
		         {
			         case VK_ESCAPE:
				         PostQuitMessage(0);
				         break;
		         }
            break;

         default:
            break;
      }

   // What this does is pass all of the unhandled messages to DefWindowProc
   return (DefWindowProc(hwnd, message, wParam, lParam));
}


int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nShowCmd)
{
   MSG msg;                // A message variable.
   WNDCLASSEX windowClass; // Your Window class.
   HWND hwnd;              // The Window handle.
   bool isFinished;        // This will be used to check if the program is done or not.

   // This is the Window class.
   windowClass.cbSize = sizeof(WNDCLASSEX);        // size of the WNDCLASSEX structure.
   windowClass.style = CS_HREDRAW | CS_VREDRAW;    // style of the window.
   windowClass.lpfnWndProc = WndProc;              // Address to the windows procedure.
   windowClass.cbClsExtra = 0;                     // Extra class information.
   windowClass.cbWndExtra = 0;                     // Extra window information.
   windowClass.hInstance = hInstance;              // Handle of application Instance.
   windowClass.hIcon = LoadIcon(NULL, IDI_APPLICATION);// Handle of application Icon.
   windowClass.hCursor = LoadCursor(NULL, IDC_ARROW);// mouse cursor
   windowClass.hbrBackground = NULL;               // background color.
   windowClass.lpszMenuName = NULL;                // name of the main menu.
   windowClass.lpszClassName = "UltimateGameProgrammingClass";// window class name.
   windowClass.hIconSm = LoadIcon(NULL, IDI_APPLICATION);// icon when minimized.

   // You must register you class with Windows.
   if(!RegisterClassEx(&windowClass))
      return 0;

   // Create the window.
   hwnd = CreateWindowEx(NULL,// The extended window style.
                        "UltimateGameProgrammingClass",// window Class name.
                        "Rotating A 3D Cube - by the Programming Ace.",// window name.
                        WS_OVERLAPPEDWINDOW | WS_VISIBLE |// The window style.
                        WS_SYSMENU | WS_CLIPCHILDREN |// window style.
                        WS_CLIPSIBLINGS,// window style.
                        100, 100,// window x, y coordinate.
                        640, 480,// window width and height.
                        NULL,// handle to parent window.
                        NULL,// menu.
                        hInstance,// handle to app instance.
                        NULL);   // pointer to window creation data.
   
   // If there was an error with creating the window, then close the program.
   if(!hwnd) return 0;

   ShowWindow(hwnd, SW_SHOW); // This shows the window.
   UpdateWindow(hwnd);        // This forces a paint message.

   isFinished = false;        // False = running, True = not running.

   if(!InitializeGL()) isFinished = true;

   // This is the messsage loop.
   while(!isFinished)
      {
         if(PeekMessage(&msg, 0, 0, 0, PM_REMOVE))
            {
               // If a quit message is received then stop rendering and quit the app.
               if(msg.message == WM_QUIT)
                  {
                     isFinished = true;
                  }

               TranslateMessage(&msg);  // Translate any messages.
               DispatchMessage(&msg);   // Dispatch any messages.
            }
         else
            {
               RenderScene();          // Render a frame.
            }
      }
   
   return (int)msg.wParam;// End of the program.
}


bool InitializeGL()
{
   glClearColor(0.0f, 0.0f, 0.0f, 0.0f);  // Clear the screen to black.

   glShadeModel(GL_SMOOTH);               // Smooth shading in our scenes.
   glEnable(GL_DEPTH_TEST);               // Enable desth testing.
   glEnable(GL_LIGHTING);                 // Enable lighting.


   // Here we will set up a spotlight light source in our scene.  This light will shine on
   // the objects giving them a nicer look than a plain dull color.  This stuff has nothing
   // to do with the demo it self but only to make things look better.
   glLightfv(GL_LIGHT0, GL_DIFFUSE, diffuseLight);
   glLightfv(GL_LIGHT0, GL_SPECULAR, specularLight);
   glLightfv(GL_LIGHT0, GL_POSITION, LightPosition);
   glLightf(GL_LIGHT0, GL_SPOT_CUTOFF, 10.0f);
   glLightf(GL_LIGHT0, GL_SPOT_EXPONENT, 75.0f);

   // Enable our light.
   glEnable(GL_LIGHT0);

   // Set up the material information for our objects.  Again this is just for show.
   glEnable(GL_COLOR_MATERIAL);
   glColorMaterial(GL_FRONT, GL_AMBIENT_AND_DIFFUSE);
   glMaterialfv(GL_FRONT, GL_SPECULAR, specularLight);
   glMateriali(GL_FRONT, GL_SHININESS, 128);

   // If all went well we return true.
   return true;
}


void RenderScene()
{
   glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);   // Clears the screen.
   glLoadIdentity();                   //Reset modelview matrix for new frame.

   // This will check the angle and make sure you can't go over 360 degrees.
   angle = angle + 0.1f;

   if(angle >= 360.0f) angle = 0.0f;

   glTranslatef(0.0f, 0.0f, -5.0f);    // This says move back 5 units.
   glRotatef(angle, 0.0f, 1.0f, 1.0f); // This rotates our scene each frame.

   glColor3f(1.0f, 1.0f, 1.0f);        // Color of the cube.

   // Now to draw a cube.  We will use auxSolidCube().
   auxSolidCube(2.0f);
   
   // Display the new frame.
   SwapBuffers(g_HDC);
}


// Recap:

// Now wasn't that fun.  To rotate an object, all you need to do is call glRotatef()
// with the rotation angle in degrees and axis that you want to rotate on.


// Copyright June 2004
// All Rights Reserved!
// Allen Sherrod
// ProgrammingAce@UltimateGameProgramming.com
// www.UltimateGameProgramming.com